/*
 * common WRN CPU definitions and routines
 *
 * Copyright (C) 2013-2016 CERN (www.cern.ch)
 * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
 * Author: Federico Vaga <federico.vaga@cern.ch>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#ifndef __TRTL_RT_COMMON_H
#define __TRTL_RT_COMMON_H

/**
 * @defgroup trtl-rt-common Common Utilities
 * Collection of general purpose utilities.
 * @{
 */

#include <stdint.h>
#include <stdio.h>

#include "hw/mockturtle_cpu_lr.h"

#include "pp-printf.h"

/**
 * Dedicated Peripheral base address
 */
#define CPU_DP_BASE 0x200000

/**
 * CPU Local Registers base address
 */
#define CPU_LR_BASE 0x100000


/**
 * Read from the Dedicated Peripheral
 * @param[in] reg register offset within the Dedicated Peripheral
 * @return the value fromt the register
 */
static inline uint32_t dp_readl(uint32_t reg)
{
	return *(volatile uint32_t *) ( reg + CPU_DP_BASE );
}


/**
 * Write to the Dedicated Peripheral
 * @param[in] value value to write
 * @param[in] reg register offset within the Dedicated Peripheral
 */
static inline void dp_writel(uint32_t value, uint32_t reg)
{
	*(volatile uint32_t *) ( reg + CPU_DP_BASE ) = value;
}


/**
 * Read from the CPU Local Registers
 * @param[in] reg register offset within the Local Registers
 * @return the value fromt the register
 */
static inline uint32_t lr_readl(uint32_t reg)
{
	return *(volatile uint32_t *) ( reg + CPU_LR_BASE );
}


/**
 * Write to the CPU Local Registers
 * @param[in] value value to write
 * @param[in] reg register offset within the Local Registers
 */
static inline void lr_writel(uint32_t value, uint32_t reg)
{
	*(volatile uint32_t *) ( reg + CPU_LR_BASE ) = value;
}


/**
 * Set a bit in the CPU GPIO Register
 * @param[in] pin GPIO pin to set
 */
static inline void gpio_set(int pin)
{
	lr_writel ((1 << pin), WRN_CPU_LR_REG_GPIO_SET);
}


/**
 * Clear a bit in the CPU GPIO Register
 * @param[in] pin GPIO pin to clear
 */
static inline void gpio_clear(int pin)
{
	lr_writel ((1 << pin), WRN_CPU_LR_REG_GPIO_CLEAR);
}


/**
 * Wait n cycles
 * fixme: use Timing Unit
 * @param[in] n number of cycles to wait
 */
static inline void delay(int n)
{
	int i;

	for(i = 0; i < n; i++)
		asm volatile("nop");
}


extern int puts(const char *p);

/**@}*/

#endif
